package it.polimi.ingsw.gc14.Model.GamePackage;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class BoardTest {

    @Test
    void getSlotList() {
        //Board board = new Board(3);
        //assertEquals("Ciao", board.getSlotList().toString());
    }

    @Test
    void getPendingEvents() {
    }

    @Test
    void containsUpperTribeCard() {
    }

    @Test
    void removeUpperTribeCard() {
    }

    @Test
    void containsLowerTribeCard() {
    }

    @Test
    void removeLowerTribeCard() {
    }

    @Test
    void containsUpperBuildingCard() {
    }

    @Test
    void removeUpperBuildingCard() {
    }

    @Test
    void containsLowerBuildingCard() {
    }

    @Test
    void removeLowerBuildingCard() {
    }

    @Test
    void nextRound() {
    }
}